"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
require("@samnbuk/react_db_client.testing.enzyme-setup");
const react_1 = __importDefault(require("react"));
const enzyme_1 = require("enzyme");
const test_utils_1 = require("react-dom/test-utils");
const constants_client_types_1 = require("@react_db_client/constants.client-types");
const components_search_and_select_dropdown_1 = require("@react_db_client/components.search-and-select-dropdown");
const field_object_ref_1 = require("./field-object-ref");
const compositions = __importStar(require("./field-object-ref.composition"));
const updateFormData = jest.fn();
const asyncGetDocuments = jest.fn();
const handleSelect = jest.fn();
const demoValue = {
    _id: 'demoitem',
    uid: 'demoitem',
    name: 'demoitem',
    label: 'demoitem',
};
const defaultProps = {
    uid: 'demoid',
    unit: 'demounit',
    updateFormData,
    handleSelect,
    asyncGetDocuments,
    value: demoValue,
    multiple: false,
    required: false,
    labelField: 'name',
    collection: 'collection',
};
describe('FieldObjectRef', () => {
    /* SETUP */
    beforeAll(() => {
        jest.useFakeTimers();
    });
    afterAll(() => {
        jest.useRealTimers();
    });
    beforeEach(() => {
        handleSelect.mockClear();
        updateFormData.mockClear();
        asyncGetDocuments.mockClear();
    });
    /* Tests */
    test('Renders', () => {
        (0, enzyme_1.shallow)(react_1.default.createElement(field_object_ref_1.FieldObjectRef, Object.assign({}, defaultProps)));
    });
    test('Matches Snapshot', () => {
        const component = (0, enzyme_1.shallow)(react_1.default.createElement(field_object_ref_1.FieldObjectRef, Object.assign({}, defaultProps)));
        const tree = component.debug();
        expect(tree).toMatchSnapshot();
    });
    describe('Compositions', () => {
        Object.entries(compositions).forEach(([name, Composition]) => {
            test(name, () => {
                (0, enzyme_1.mount)(react_1.default.createElement(Composition, null));
            });
        });
    });
    describe('Unit tests', () => {
        let component;
        beforeEach(() => {
            component = (0, enzyme_1.mount)(react_1.default.createElement(field_object_ref_1.FieldObjectRef, Object.assign({}, defaultProps)));
        });
        describe('shows a search select field', () => {
            test('should show search and select component', () => {
                expect(component.find(components_search_and_select_dropdown_1.SearchAndSelectDropdown)).toBeTruthy();
            });
        });
        describe('handles select', () => {
            test('should call update fn when search component returns selection', () => {
                const searchComponent = component.find(components_search_and_select_dropdown_1.SearchAndSelectDropdown);
                const selectedId = 'demoid';
                const selectedData = {};
                searchComponent.props().handleSelect(selectedId, selectedData);
                expect(updateFormData).toHaveBeenCalledWith(defaultProps.uid, selectedData);
            });
        });
        describe('Passes props to search and select', () => {
            test('should provide default search function if searchFn override is null', () => {
                const searchAndSelectDropdown = component.find(components_search_and_select_dropdown_1.SearchAndSelectDropdown);
                expect(searchAndSelectDropdown.props()).toMatchSnapshot();
            });
        });
        describe('Searching', () => {
            test('Entering search input should call async get documents', () => __awaiter(void 0, void 0, void 0, function* () {
                // TODO: Complete this
                const searchField = component.find('input').find('[type="text"]').first();
                (0, test_utils_1.act)(() => {
                    searchField.simulate('focus');
                    component.update();
                });
                const searchValue = 'Search Val';
                yield (0, test_utils_1.act)(() => __awaiter(void 0, void 0, void 0, function* () {
                    searchField.simulate('change', { target: { value: searchValue } });
                    component.update();
                    jest.runOnlyPendingTimers();
                    component.update();
                    jest.runOnlyPendingTimers();
                }));
                const collection = defaultProps.collection;
                const filters = [
                    new constants_client_types_1.FilterObjectClass({
                        uid: 'search',
                        field: defaultProps.labelField,
                        value: searchValue,
                    }),
                ];
                const schema = '_id';
                const sortBy = defaultProps.labelField;
                expect(asyncGetDocuments).toHaveBeenCalledWith(collection, filters, schema, sortBy);
            }));
        });
    });
});
//# sourceMappingURL=field-object-ref.test.js.map